@@ -34,8 +34,9 @@
 
 // HEADER FILES ------------------------------------------------------------
 
-#include <string>
 #include <algorithm>
+#include <stdexcept>
+#include <string>
 #include <assert.h>
 #include "zmusic/zmusic_internal.h"
 #include "zmusic/musinfo.h"
@@ -87,7 +88,7 @@ public:
 	int ServiceEvent();
 	void SetMIDISource(MIDISource* _source);
 	bool ServiceStream(void* buff, int len) override;
-	SoundStreamInfo GetStreamInfo() const override;
+	SoundStreamInfoEx GetStreamInfoEx() const override;
 
 	int GetDeviceType() const override;
 
@@ -446,10 +447,10 @@ bool MIDIStreamer::InitPlayback()
 	}
 }
 
-SoundStreamInfo MIDIStreamer::GetStreamInfo() const
+SoundStreamInfoEx MIDIStreamer::GetStreamInfoEx() const
 {
-	if (MIDI) return MIDI->GetStreamInfo();
-	else return { 0, 0, 0 };
+	if (MIDI) return MIDI->GetStreamInfoEx();
+	else return {};
 }
 
 //==========================================================================
@@ -814,7 +815,7 @@ int MIDIStreamer::FillBuffer(int buffer_num, int max_events, uint32_t max_time)
 	if (InitialPlayback)
 	{
 		InitialPlayback = false;
-		// Send the GS System Reset SysEx message.
+		// Send the GM System Enable SysEx message.
 		events[0] = 0;								// dwDeltaTime
 		events[1] = 0;								// dwStreamID
 		events[2] = (MEVENT_LONGMSG << 24) | 6;		// dwEvent
@@ -822,6 +823,15 @@ int MIDIStreamer::FillBuffer(int buffer_num, int max_events, uint32_t max_time)
 		events[4] = MAKE_ID(0x01, 0xf7, 0x00, 0x00);	// dwParms[1]
 		events += 5;
 
+		// Send the GS DT1 MODE SET GS Reset SysEx message.
+		events[0] = 0;									// dwDeltaTime
+		events[1] = 0;									// dwStreamID
+		events[2] = (MEVENT_LONGMSG << 24) | 11;		// dwEvent
+		events[3] = MAKE_ID(0xf0, 0x41, 0x7f, 0x42);	// dwParms[0]
+		events[4] = MAKE_ID(0x12, 0x40, 0x00, 0x7f);	// dwParms[1]
+		events[5] = MAKE_ID(0x00, 0x41, 0xf7, 0x00);	// dwParms[2]
+		events += 6;
+
 		// Send the full master volume SysEx message.
 		events[0] = 0;								// dwDeltaTime
 		events[1] = 0;								// dwStreamID
@@ -1021,7 +1031,7 @@ DLL_EXPORT zmusic_bool ZMusic_MIDIDumpWave(ZMusic_MidiSource source, EMidiDevice
 {
 	try
 	{
-		MIDIStreamer me(devtype, devarg);
+		MIDIStreamer me(devtype, devarg ? devarg : "");
 		me.SetMIDISource(source);
 		me.DumpWave(outname, subsong, samplerate);
 		return true;
