@@ -2,7 +2,7 @@
  * libOPNMIDI is a free Software MIDI synthesizer library with OPN2 (YM2612) emulation
  *
  * MIDI parser and player (Original code from ADLMIDI): Copyright (c) 2010-2014 Joel Yliluoma <bisqwit@iki.fi>
- * OPNMIDI Library and YM2612 support:   Copyright (c) 2017-2020 Vitaly Novichkov <admin@wohlnet.ru>
+ * OPNMIDI Library and YM2612 support:   Copyright (c) 2017-2022 Vitaly Novichkov <admin@wohlnet.ru>
  *
  * Library is based on the ADLMIDI, a MIDI player for Linux and Windows with OPL3 emulation:
  * http://iki.fi/bisqwit/source/adlmidi.html
@@ -218,6 +218,7 @@ OPN2::OPN2() :
     m_masterVolume(MasterVolumeDefault),
     m_musicMode(MODE_MIDI),
     m_volumeScale(VOLUME_Generic),
+    m_channelAlloc(OPNMIDI_ChanAlloc_AUTO),
     m_lfoEnable(false),
     m_lfoFrequency(0),
     m_chipFamily(OPNChip_OPN2)
@@ -226,6 +227,7 @@ OPN2::OPN2() :
     m_insBankSetup.lfoEnable = false;
     m_insBankSetup.lfoFrequency = 0;
     m_insBankSetup.chipType = OPNChip_OPN2;
+    m_insBankSetup.mt32defaults = false;
 
     // Initialize blank instruments banks
     m_insBanks.clear();
@@ -412,6 +414,9 @@ void OPN2::touchNote(size_t c,
         volume = velocity * channelVolume * channelExpression;
         //volume = volume * m_masterVolume / (127 * 127 * 127) / 2;
         volume = (volume * m_masterVolume) / 4096766;
+
+        if(volume > 0)
+            volume += 64;//OPN has 0~127 range. As 0...63 is almost full silence, but at 64 to 127 is very closed to OPL3, just add 64.
     }
     break;
 
@@ -538,6 +543,7 @@ void OPN2::setVolumeScaleModel(OPNMIDI_VolumeModels volumeModel)
 {
     switch(volumeModel)
     {
+    default:
     case OPNMIDI_VolumeModel_AUTO://Do nothing until restart playing
         break;
 
@@ -600,6 +606,7 @@ void OPN2::reset(int emulator, unsigned long PCM_RATE, OPNFamily family, void *a
     if(emulator == OPNMIDI_VGM_DUMPER && (m_numChips > 2))
         m_numChips = 2;// VGM Dumper can't work in multichip mode
 #endif
+    m_chips.clear();
     m_chips.resize(m_numChips, AdlMIDI_SPtr<OPNChipBase>());
 
 #ifdef OPNMIDI_MIDI2VGM
@@ -611,7 +618,7 @@ void OPN2::reset(int emulator, unsigned long PCM_RATE, OPNFamily family, void *a
 
     for(size_t i = 0; i < m_chips.size(); i++)
     {
-        OPNChipBase *chip;
+        OPNChipBase *chip = NULL;
 
         switch(emulator)
         {
@@ -655,7 +662,7 @@ void OPN2::reset(int emulator, unsigned long PCM_RATE, OPNFamily family, void *a
 #endif
 #ifdef OPNMIDI_MIDI2VGM
         case OPNMIDI_VGM_DUMPER:
-            chip = new VGMFileDumper(family);
+            chip = new VGMFileDumper(family, i, (i == 0 ? NULL : m_chips[0].get()));
             if(i == 0)//Set hooks for first chip only
             {
                 m_loopStartHook = &VGMFileDumper::loopStartHook;
@@ -666,11 +673,14 @@ void OPN2::reset(int emulator, unsigned long PCM_RATE, OPNFamily family, void *a
             break;
 #endif
         }
+
         m_chips[i].reset(chip);
         chip->setChipId(static_cast<uint32_t>(i));
         chip->setRate(static_cast<uint32_t>(PCM_RATE), chip->nativeClockRate());
+
         if(m_runAtPcmRate)
             chip->setRunningAtPcmRate(true);
+
 #if defined(ADLMIDI_AUDIO_TICK_HANDLER)
         chip->setAudioTickHandlerInstance(audioTickHandler);
 #endif
@@ -685,18 +695,18 @@ void OPN2::reset(int emulator, unsigned long PCM_RATE, OPNFamily family, void *a
     uint8_t regLFOSetup = (m_lfoEnable ? 8 : 0) | (m_lfoFrequency & 7);
     m_regLFOSetup = regLFOSetup;
 
-    for(size_t card = 0; card < m_numChips; ++card)
+    for(size_t chip = 0; chip < m_numChips; ++chip)
     {
-        writeReg(card, 0, 0x22, regLFOSetup);//push current LFO state
-        writeReg(card, 0, 0x27, 0x00);  //set Channel 3 normal mode
-        writeReg(card, 0, 0x2B, 0x00);  //Disable DAC
+        writeReg(chip, 0, 0x22, regLFOSetup);//push current LFO state
+        writeReg(chip, 0, 0x27, 0x00);  //set Channel 3 normal mode
+        writeReg(chip, 0, 0x2B, 0x00);  //Disable DAC
         //Shut up all channels
-        writeReg(card, 0, 0x28, 0x00 ); //Note Off 0 channel
-        writeReg(card, 0, 0x28, 0x01 ); //Note Off 1 channel
-        writeReg(card, 0, 0x28, 0x02 ); //Note Off 2 channel
-        writeReg(card, 0, 0x28, 0x04 ); //Note Off 3 channel
-        writeReg(card, 0, 0x28, 0x05 ); //Note Off 4 channel
-        writeReg(card, 0, 0x28, 0x06 ); //Note Off 5 channel
+        writeReg(chip, 0, 0x28, 0x00); //Note Off 0 channel
+        writeReg(chip, 0, 0x28, 0x01); //Note Off 1 channel
+        writeReg(chip, 0, 0x28, 0x02); //Note Off 2 channel
+        writeReg(chip, 0, 0x28, 0x04); //Note Off 3 channel
+        writeReg(chip, 0, 0x28, 0x05); //Note Off 4 channel
+        writeReg(chip, 0, 0x28, 0x06); //Note Off 5 channel
     }
 
     silenceAll();
